<?php namespace Premmerce\Wishlist\Frontend;

use Premmerce\Wishlist\Models\WishlistModel;
use Premmerce\Wishlist\WishlistStorage;


class WishlistFunctions {


    /**
     * @var WishlistModel
     */
    private $model;

    /**
     * @var WishlistStorage
     */
    private $storage;


    private static $_instance = null;

    private function __construct(){
        $this->model = new WishlistModel();
        $this->storage = new WishlistStorage($this->model);
    }

    /**
     * Static class initialization
     *
     * @return WishlistFunctions
     */
    public static function getInstance() {
        if (is_null(self::$_instance)) {
            self::$_instance = new self();
        }

        return self::$_instance;
    }

    public function checkInWishlist($productId) {
        $wishlists = [];

        if (is_user_logged_in()) {
            $wishlists = $this->model->getWishlistsByUserId(get_current_user_id());
        } elseif ($this->storage->cookieIsSet()) {
            $wishlists = $this->model->getWishlistsByKeys($this->storage->cookieGet());
        }

        foreach ($wishlists as $wl) {
            if (in_array($productId,$wl['products'])) {
                return true;
            }
        }

        return false;
    }

    public function wishlistTotal() {
        $wishlist = [];
        $countProducts = 0;

        if (is_user_logged_in()) {
            $wishlist = $this->model->getWishlistsByUserId(get_current_user_id());
        } elseif ($this->storage->cookieIsSet()) {
            $wishlist = $this->model->getWishlistsByKeys($this->storage->cookieGet());
        }

        if ($wishlist) {
            foreach ($wishlist as $wl) {
                $countProducts += count(array_filter($wl['products']));
            }
        }

        return $countProducts;
    }

    public function isProductInWishlist($id, $key) {
        $wishlist = $this->model->getWishlistsByProductId($id);

        if ($wishlist[0]) {
            return $wishlist[0]['wishlist_key'] == $key;
        }

        return false;
    }

    public function getWishlistKeyByProductId($id) {
        $wishlist = $this->model->getWishlistsByProductId($id);

        if ($wishlist[0]) {
            return $wishlist[0]['wishlist_key'];
        }

        return '';
    }

    public function getWishlists() {
        return $this->model->getWishlists();
    }
}